@@ -10,6 +10,7 @@
  */
 #include <string.h>
 #include <pthread.h>
+#include <assert.h>
 #include "usbi.h"
 
 #define USB_EP_TYPE_MASK 0x03
@@ -94,11 +95,11 @@ int32_t openusb_claim_interface(openusb_dev_handle_t dev, uint8_t ifc,
 	if (usbi_fetch_and_parse_descriptors(hdev) != 0) {
 		return OPENUSB_BADARG;
 	}
-	
+
 	pthread_mutex_lock(&hdev->lock);
 	/* check if this is a valid interface */
 	if ((ifc>= USBI_MAXINTERFACES) ||
-	    (ifc >= hdev->idev->desc.configs[hdev->config_value-1].
+	    (ifc >= hdev->idev->desc.configs[hdev->idev->cur_config_index].
 	    		num_interfaces)) {
 
 		usbi_debug(hdev->lib_hdl, 1, "interface %d not valid",
@@ -195,7 +196,16 @@ int32_t openusb_set_altsetting(openusb_dev_handle_t dev, uint8_t ifc,
 	}
 
 	pthread_mutex_lock(&hdev->lock);
-	pcfg=&idev->desc.configs[hdev->config_value - 1];
+
+
+	if (idev->cur_config_index < 0) {
+		usbi_debug(hdev->lib_hdl, 1, "config value = %d\n",
+			idev->cur_config_value);
+
+		return (OPENUSB_PARSE_ERROR);
+	} else {
+		pcfg=&idev->desc.configs[idev->cur_config_index];
+	}
 
 	/* not valid interface, or not claimed, or not valid alt */
 	if (ifc > pcfg->num_interfaces || ifc > USBI_MAXINTERFACES
@@ -323,7 +333,7 @@ int32_t usbi_get_config_desc(struct usbi_dev_handle *devh, int cfg, char **cfgbu
 	openusb_parse_data("bbw", (unsigned char *)buf, 8, &cfg_desc,
 			sizeof(cfg_desc), &count);
 
-	newbuf = malloc(cfg_desc.wTotalLength);
+	newbuf = calloc(cfg_desc.wTotalLength, 1);
 	if (!newbuf) {
 		usbi_debug(NULL, 1, "no memory");
 		return OPENUSB_NO_RESOURCES;
@@ -395,6 +405,9 @@ int32_t check_req_valid(openusb_request_handle_t req,
 		if (openusb_is_interface_claimed(dev->handle, ifc) == 1) {
 			return 0;
 		} else {
+			usbi_debug(dev->lib_hdl, 1, "interface %d not claimed",
+				ifc);
+
 			return -1;
 		}
 	}
@@ -510,19 +523,74 @@ int32_t openusb_xfer_wait(openusb_request_handle_t req)
 	struct usbi_dev_handle *dev=NULL;
 	int32_t	io_pattern;
 
-	usbi_debug(NULL, 4, "Begin: ifc=%d ept=%x type=%d", req->interface,
-		req->endpoint, req->type);
-
 	if(!req) {
 		usbi_debug(NULL, 1, "Invalid request");
 		return OPENUSB_BADARG;
 	}
 
+	usbi_debug(NULL, 4, "Begin: ifc=%d ept=%x type=%d", req->interface,
+		req->endpoint, req->type);
+
 	dev = usbi_find_dev_handle(req->dev); 
 	if (!dev) {
 		usbi_debug(NULL, 1, "Can't find device handle:%llu",req->dev);
 		return OPENUSB_INVALID_HANDLE;
 	}
+
+	/* Make sure the request is not too large (if the max size is zero then
+	 * there is no maximum size */
+	if (dev->idev->bus->max_xfer_size[req->type] != 0) {
+		switch(req->type) {
+		default:
+			usbi_debug(dev->lib_hdl, 1,
+				"Invalid request type: %d", req->type);
+			return (OPENUSB_BADARG);
+				
+		case USB_TYPE_CONTROL:
+			if (req->req.ctrl->length >
+				dev->idev->bus->max_xfer_size[req->type]) {
+				usbi_debug(dev->lib_hdl, 1,
+				    "Request too large (%u),"
+				    " max_xfer_size=%u",
+				    req->req.ctrl->length,
+				    dev->idev->bus->max_xfer_size[req->type]);
+
+				return (OPENUSB_IO_REQ_TOO_BIG);
+			}
+			break;
+
+		case USB_TYPE_INTERRUPT:
+			if (req->req.intr->length >
+				dev->idev->bus->max_xfer_size[req->type]) {
+				usbi_debug(dev->lib_hdl, 1,
+				    "Request too large (%u), max_xfer_size=%u",
+				    req->req.intr->length,
+				    dev->idev->bus->max_xfer_size[req->type]);
+				
+				return (OPENUSB_IO_REQ_TOO_BIG);
+			}
+			break;
+
+		case USB_TYPE_BULK:
+			if (req->req.bulk->length >
+				dev->idev->bus->max_xfer_size[req->type]) {
+				usbi_debug(dev->lib_hdl, 1,
+				    "Request too large (%u), max_xfer_size=%u",
+				    req->req.bulk->length,
+				    dev->idev->bus->max_xfer_size[req->type]);
+
+				return (OPENUSB_IO_REQ_TOO_BIG);
+			}
+			break;
+				
+		case USB_TYPE_ISOCHRONOUS:
+			/*
+			 * FIXME: is there a good way to check the
+			 * length here?
+			 */
+			break;
+		} /* end switch(req->type) { */
+	} /* end if (dev->idev->bus->max_xfer_size[req->type] != 0) { */
 	
 	if (check_req_valid(req, dev) < 0) {
 		usbi_debug(dev->lib_hdl, 1, "Not a valid request");
@@ -704,19 +772,74 @@ int32_t openusb_xfer_aio(openusb_request_handle_t req)
 	int32_t timeout;
 	struct usbi_handle *plib;
 
-	usbi_debug(NULL, 4, "Begin: ifc=%d ept=%x type=%d", req->interface,
-						 req->endpoint, req->type);
-	
 	if (!req) {
 		return OPENUSB_BADARG;
 	}
 
+	usbi_debug(NULL, 4, "Begin: ifc=%d ept=%x type=%d",
+		req->interface, req->endpoint, req->type);
+
 	dev = usbi_find_dev_handle(req->dev); 
 	if (!dev) {
 		usbi_debug(NULL, 1, "Can't find device");
 		return OPENUSB_BADARG;
 	}
 
+	/*
+	 * Make sure the request is not too large (if the max size
+	 * is zero then there is no maximum size
+	 */
+	if (dev->idev->bus->max_xfer_size[req->type] != 0) {
+		switch(req->type) {
+		default:
+			usbi_debug(dev->lib_hdl, 1,
+			    "Invalid request type: %d", req->type);
+
+			return (OPENUSB_BADARG);
+				
+		case USB_TYPE_CONTROL:
+			if (req->req.ctrl->length >
+			    dev->idev->bus->max_xfer_size[req->type]) {
+				usbi_debug(dev->lib_hdl, 1,
+				"Request too large (%u), max_xfer_size=%u",
+				req->req.ctrl->length,
+				dev->idev->bus->max_xfer_size[req->type]);
+
+				return (OPENUSB_IO_REQ_TOO_BIG);
+			}
+			break;
+
+		case USB_TYPE_INTERRUPT:
+			if (req->req.intr->length >
+			    dev->idev->bus->max_xfer_size[req->type]) {
+				usbi_debug(dev->lib_hdl, 1,
+				"Request too large (%u), max_xfer_size=%u",
+				req->req.intr->length,
+				dev->idev->bus->max_xfer_size[req->type]);
+
+				return (OPENUSB_IO_REQ_TOO_BIG);
+			}
+
+			break;
+
+		case USB_TYPE_BULK:
+			if (req->req.bulk->length >
+			    dev->idev->bus->max_xfer_size[req->type]) {
+				usbi_debug(dev->lib_hdl, 1,
+				"Request too large (%u), max_xfer_size=%u",
+				req->req.bulk->length,
+				dev->idev->bus->max_xfer_size[req->type]);
+
+				return (OPENUSB_IO_REQ_TOO_BIG);
+			}
+			break;
+				
+		case USB_TYPE_ISOCHRONOUS:
+		/* FIXME: is there a good way to check the length here? */
+			break;
+		} /* end switch(req->type) { */
+	} /* end if (dev->idev->bus->max_xfer_size[req->type] != 0) { */
+
 	ret = check_req_valid(req, dev);
 	if (ret < 0) {
 		usbi_debug(dev->lib_hdl, 1, "Invalid request");
@@ -750,7 +873,7 @@ int32_t openusb_xfer_aio(openusb_request_handle_t req)
 
 		usbi_free_io(io);
 		return ret;
-	}	
+	}
 
 	usbi_debug(NULL, 4, "End");
 
@@ -1014,7 +1137,7 @@ int32_t multi_req_callback(openusb_request_handle_t req)
 			isoc->pkts[idx].num_packets;
 	}
 
-	result = malloc(len);
+	result = calloc(len, 1);
 	if (!result) {
 		return OPENUSB_NO_RESOURCES;
 	}
@@ -1099,7 +1222,7 @@ loop:
 	for(i = 0; i< req_num; i++) {
 		usbi_debug(hdev->lib_hdl, 4, "submit request %d",i);
 
-		req = malloc(sizeof(struct openusb_request_handle));
+		req = calloc(sizeof(struct openusb_request_handle), 1);
 		if (!req) {
 			usbi_debug(hdev->lib_hdl, 1, "No resources");
 			pthread_mutex_unlock(&mi_req->lock);
@@ -1109,7 +1232,7 @@ loop:
 		/* args is the additional argument of callback 
 		 * in openusb_request_handle_t
 		 */
-		args = malloc(sizeof(struct usbi_multi_req_args));
+		args = calloc(sizeof(struct usbi_multi_req_args), 1);
 		if (!args) {
 			usbi_debug(hdev->lib_hdl, 1, "No resources");
 			pthread_mutex_unlock(&mi_req->lock);
@@ -1143,7 +1266,7 @@ loop:
 			m_bulk = mh->req.bulk;
 
 			/* submit all the request buffers */
-			bulk = malloc(sizeof(*bulk));
+			bulk = calloc(sizeof(*bulk), 1);
 			if (!bulk) {
 				pthread_mutex_unlock(&mi_req->lock);
 				return OPENUSB_NO_RESOURCES;
@@ -1171,7 +1294,7 @@ loop:
 
 			m_intr = mh->req.intr;
 
-			intr = malloc(sizeof(*intr));
+			intr = calloc(sizeof(*intr), 1);
 			if (!intr) {
 				pthread_mutex_unlock(&mi_req->lock);
 				return OPENUSB_NO_RESOURCES;
@@ -1208,7 +1331,7 @@ loop:
 			m_isoc = mh->req.isoc;
 
 			/* submit all the request buffers */
-			isoc = malloc(sizeof(*isoc));
+			isoc = calloc(sizeof(*isoc), 1);
 			if (!isoc) {
 				free(req);
 				pthread_mutex_unlock(&mi_req->lock);
@@ -1294,13 +1417,13 @@ int32_t openusb_start(openusb_multi_request_handle_t handle)
 		return OPENUSB_BADARG;
 	}
 
-	mi_req = malloc(sizeof(struct usbi_multi_request));
+	mi_req = calloc(sizeof(struct usbi_multi_request), 1);
 	if (!mi_req) {
 		usbi_debug(hdev->lib_hdl, 1, "malloc fail");
 		return OPENUSB_NO_RESOURCES;
 	}
 
-	req = malloc(sizeof(struct openusb_request_handle));
+	req = calloc(sizeof(struct openusb_request_handle), 1);
 	if (!req) {
 		return OPENUSB_NO_RESOURCES;
 	}
