@@ -1289,9 +1289,6 @@ solaris_open(struct usbi_dev_handle *hdev)
 		
 	}
 
-	hdev->config_value = 1;
-	hdev->priv->config_index = -1;
-
 	/* open default control ep and keep it open */
 	if (usb_open_ep0(hdev) != 0) {
 
@@ -1307,6 +1304,9 @@ solaris_open(struct usbi_dev_handle *hdev)
 		return OPENUSB_SYS_FUNC_FAILURE;
 	}
 
+	idev->cur_config_index = 0;
+	idev->cur_config_value = 1;
+
 	return (OPENUSB_SUCCESS);
 }
 
@@ -1316,7 +1316,7 @@ solaris_get_configuration(struct usbi_dev_handle *hdev, uint8_t *cfg)
 	if(!hdev) {
 		return (OPENUSB_NOT_SUPPORTED);
 	}
-	*cfg = hdev->config_value;
+	*cfg = hdev->idev->cur_config_value;
 
 	return OPENUSB_SUCCESS;
 }
@@ -1329,8 +1329,8 @@ solaris_set_configuration(struct usbi_dev_handle *hdev, uint8_t cfg)
 		return (OPENUSB_NOT_SUPPORTED);
 	}
 
-	hdev->config_value = cfg;
-	hdev->priv->config_index = hdev->config_value - 1;
+	hdev->idev->cur_config_value = cfg;
+	hdev->idev->cur_config_index = usbi_get_cfg_index_by_value(hdev, cfg);
 
 	return OPENUSB_SUCCESS;
 }
@@ -1397,7 +1397,7 @@ solaris_claim_interface(struct usbi_dev_handle *hdev, uint8_t interface,
 	struct usbi_device *idev = hdev->idev;
 	int index;
 
-	index = hdev->config_value - 1;
+	index = hdev->idev->cur_config_index;
 
 	/* already claimed this interface */
 	if (idev->priv->info.claimed_interfaces[interface] == hdev) {
@@ -1475,6 +1475,8 @@ usb_close_all_eps(struct usbi_dev_handle *hdev)
 			(void) close(hdev->priv->eps[i].statfd);
 			hdev->priv->eps[i].statfd = -1;
 		}
+
+		hdev->priv->epflags[i] &= ~ISOC_IN_INITED;
 	}
 }
 
@@ -1742,8 +1744,7 @@ usb_check_device_and_status_open(struct usbi_dev_handle *hdev,uint8_t ifc,
 
 	ep_index = usb_ep_index(ep_addr);
 	
-	if ((hdev->config_value == -1) || 
-		(hdev->claimed_ifs[ifc].clm == -1) ||
+	if ((hdev->claimed_ifs[ifc].clm == -1) ||
 		(hdev->claimed_ifs[ifc].altsetting == -1)) {
 
 		usbi_debug(hdev->lib_hdl, 1, "interface not claimed");
@@ -1778,9 +1779,9 @@ usb_check_device_and_status_open(struct usbi_dev_handle *hdev,uint8_t ifc,
 	}
 	
 	/* create filename */
-	if (hdev->priv->config_index > 0) {
+	if (hdev->idev->cur_config_index > 0) {
 		(void) snprintf(cfg_num, sizeof (cfg_num), "cfg%d",
-		    hdev->config_value);
+		    hdev->idev->cur_config_value);
 	} else {
 		(void) memset(cfg_num, 0, sizeof (cfg_num));
 	}
@@ -2124,7 +2125,6 @@ solaris_submit_intr(struct usbi_dev_handle *hdev, struct usbi_io *io)
 	return (ret);
 }
 
-
 /*
  * isoc data = isoc pkt header + isoc pkt desc + isoc pkt desc ...+ data
  */
@@ -2237,13 +2237,35 @@ solaris_submit_isoc(struct usbi_dev_handle *hdev, struct usbi_io *io)
 	 * simutaneously, or some thread close the device while another is 
 	 * accessing it? 
 	 */
-
-	/* do isoc OUT xfer or init polling for isoc IN xfer */
-	ret = usb_do_io(hdev->priv->eps[ep_index].datafd,
+	
+	
+	if ((ep_addr & USB_ENDPOINT_DIR_MASK) == 0) {
+		/* do isoc OUT xfer*/
+		ret = usb_do_io(hdev->priv->eps[ep_index].datafd,
 		hdev->priv->eps[ep_index].statfd, (char *)buf, len, WRITE,
 		&isoc->isoc_status);
 
+	} else if ((ep_addr & USB_ENDPOINT_DIR_MASK) &&
+		((hdev->priv->epflags[ep_index] & ISOC_IN_INITED) == 0)) {
+		/*
+		 * Write request information to initialize polling
+		 * for isoc IN xfer. Kernel will start ISOC IN polling
+		 * immediately after set parameters successfully.
+		 */
+		ret = usb_do_io(hdev->priv->eps[ep_index].datafd,
+			hdev->priv->eps[ep_index].statfd, (char *)buf,
+			len, WRITE, &isoc->isoc_status);
+		hdev->priv->epflags[ep_index] |= ISOC_IN_INITED;
+	} else {
+		/* isoc IN endpoint, polling already started. i.e. Don't
+		 * write ISOC IN request twice, otherwise the kernel will
+		 * return failure
+		 */
+		ret = 0;
+	}
+
 	if (ret < 0) {
+		fprintf(stderr, "write isoc failed %d\n", ret);
 		usbi_debug(hdev->lib_hdl, 1, "write isoc ep failed %d TID=%d",
 			ret, pthread_self());
 
@@ -2330,8 +2352,10 @@ solaris_submit_isoc(struct usbi_dev_handle *hdev, struct usbi_io *io)
 		for(pkt = 0; pkt < n_pkt; pkt++) {
 			usbi_debug(hdev->lib_hdl, 4, "packet: %d, len: %d", pkt,
 					packet[pkt].length);
+
 			memcpy(packet[pkt].payload, p, packet[pkt].length); 
 			p += packet[pkt].length;
+
 			isoc->isoc_results[pkt].status = 
 				pkt_descr[pkt].isoc_pkt_status;
 			isoc->isoc_results[pkt].transferred_bytes = 
@@ -2340,12 +2364,19 @@ solaris_submit_isoc(struct usbi_dev_handle *hdev, struct usbi_io *io)
 
 		free(buf);
 
+		/*
+		 * To avoid data loss, we have to keep the ISOC IN
+		 * endpoint polling. Application must call openusb_close_device
+		 * to stop the periodic endpoints polling.
+		 */
+#if 0
 		/* we have to close this pipe to stop ISOC IN polling */
 		(void) close(hdev->priv->eps[ep_index].datafd);
 		(void) close(hdev->priv->eps[ep_index].statfd);
 
 		hdev->priv->eps[ep_index].datafd = -1;
 		hdev->priv->eps[ep_index].statfd = -1;
+#endif
 	}
 
 	pthread_mutex_unlock(&hdev->lock);
@@ -2371,7 +2402,7 @@ solaris_set_altinterface(struct usbi_dev_handle *hdev, uint8_t ifc, uint8_t alt)
 
 	usb_close_all_eps(hdev);
 	iface = ifc;
-	index = hdev->config_value - 1;
+	index = hdev->idev->cur_config_index;
 
 	/* set alt interface is implicitly done when endpoint is opened */
 	hdev->claimed_ifs[ifc].altsetting = alt;
