@@ -183,7 +183,7 @@ void event_cb(openusb_handle_t handle, openusb_devid_t devid,
 		break;
 	}
 	printf("CALLBACK: lib(%llu) device(%llu) get a event(%s) with arg=%p\n",
-		handle, devid, evstring, arg);
+		(long long)handle, (long long)devid, evstring, arg);
 }
 
 int convert_string(char *buf, usb_string_desc_t *st, int buflen)
@@ -350,14 +350,16 @@ int test_bulk_sync(openusb_handle_t devh)
 
 	ret = openusb_claim_interface(devh,0,0);
 	if (ret != 0) {
-		printf("Device(%llu) claim interface error:%s\n",devids[0],
+		printf("Device(%llu) claim interface error:%s\n",
+				(long long)devids[0],
 				openusb_strerror(ret));
 		return -1;
 	}
 
 	ret = openusb_set_altsetting(devh,0,0);
 	if (ret != 0) {
-		printf("Device(%llu) interface(0) set alt:%s\n",devids[0],
+		printf("Device(%llu) interface(0) set alt:%s\n",
+				(long long)devids[0],
 				openusb_strerror(ret));
 		return -1;
 	}
@@ -424,14 +426,16 @@ int test_intr_sync(openusb_handle_t devh, int flag)
 
 	ret = openusb_claim_interface(devh,0,0);
 	if (ret != 0) {
-		printf("Device(%llu) claim interface error:%s\n",devids[0],
+		printf("Device(%llu) claim interface error:%s\n",
+				(long long)devids[0],
 				openusb_strerror(ret));
 		return -1;
 	}
 
 	ret = openusb_set_altsetting(devh,0,0);
 	if (ret != 0) {
-		printf("Device(%llu) interface(0) set alt:%s\n",devids[0],
+		printf("Device(%llu) interface(0) set alt:%s\n",
+				(long long)devids[0],
 				openusb_strerror(ret));
 		return -1;
 	}
@@ -498,14 +502,16 @@ int test_isoc_sync(openusb_handle_t devh)
 
 	ret = openusb_claim_interface(devh,0,0);
 	if (ret != 0) {
-		printf("Device(%llu) claim interface error:%s\n",devids[0],
+		printf("Device(%llu) claim interface error:%s\n",
+				(long long)devids[0],
 				openusb_strerror(ret));
 		return -1;
 	}
 
 	ret = openusb_set_altsetting(devh,0,3);/* alt 3, depends on the fw */
 	if (ret != 0) {
-		printf("Device(%llu) interface(0) set alt:%s\n",devids[0],
+		printf("Device(%llu) interface(0) set alt:%s\n",
+				(long long)devids[0],
 				openusb_strerror(ret));
 		return -1;
 	}
@@ -593,14 +599,16 @@ int async_xfer_ctrl_test(openusb_dev_handle_t devh)
 
 	ret = openusb_claim_interface(devh,0,0);
 	if (ret != 0) {
-		printf("Device(%llu) claim interface error:%s\n",devids[0],
+		printf("Device(%llu) claim interface error:%s\n",
+				(long long)devids[0],
 				openusb_strerror(ret));
 		return -1;
 	}
 
 	ret = openusb_set_altsetting(devh,0,0);
 	if (ret != 0) {
-		printf("Device(%llu) interface(0) set alt:%s\n",devids[0],
+		printf("Device(%llu) interface(0) set alt:%s\n",
+				(long long)devids[0],
 				openusb_strerror(ret));
 		return -1;
 	}
@@ -735,7 +743,8 @@ int async_xfer_test(openusb_handle_t devh, openusb_transfer_type_t type, int fla
 
 	ret = openusb_claim_interface(devh,0,0);
 	if (ret != 0) {
-		printf("Device(%llu) claim interface error:%s\n", devids[0],
+		printf("Device(%llu) claim interface error:%s\n",
+				(long long)devids[0],
 				openusb_strerror(ret));
 		return -1;
 	}
@@ -747,7 +756,8 @@ int async_xfer_test(openusb_handle_t devh, openusb_transfer_type_t type, int fla
 	}
 
 	if (ret != 0) {
-		printf("Device(%llu) interface(0) set alt:%s\n", devids[0],
+		printf("Device(%llu) interface(0) set alt:%s\n",
+				(long long)devids[0],
 				openusb_strerror(ret));
 		return -1;
 	}
@@ -1027,14 +1037,16 @@ int multi_xfer_test(openusb_dev_handle_t devh)
 
 	ret = openusb_claim_interface(devh,0,0);
 	if (ret != 0) {
-		printf("Device(%llu) claim interface error:%s\n", devids[0],
+		printf("Device(%llu) claim interface error:%s\n",
+				(long long)devids[0],
 				openusb_strerror(ret));
 		return -1;
 	}
 
 	ret = openusb_set_altsetting(devh,0,0);
 	if (ret != 0) {
-		printf("Device(%llu) interface(0) set alt:%s\n", devids[0],
+		printf("Device(%llu) interface(0) set alt:%s\n",
+				(long long)devids[0],
 				openusb_strerror(ret));
 		return -1;
 	}
@@ -1195,7 +1207,8 @@ int advance_xfer_test(void)
 				openusb_strerror(ret));
 		goto err;
 	}
-	printf("Device(%llu) opened: %llu\n",devids[0],devh);
+	printf("Device(%llu) opened: %llu\n", (long long)devids[0],
+			(long long)devh);
 
 	ret = openusb_get_devid(devh, &devid);
 	if(ret < 0) {
@@ -1206,13 +1219,14 @@ int advance_xfer_test(void)
 
 	ret = openusb_get_lib_handle(devh,&libh);
 	if (ret != 0) {
-		printf("Get device(%llu) lib handle error:%s\n",devids[0],
+		printf("Get device(%llu) lib handle error:%s\n",
+				(long long)devids[0],
 				openusb_strerror(ret));
 		goto err;
 	}
-	printf("Lib handle = %llu\n",libh);
+	printf("Lib handle = %llu\n", (long long)libh);
 
-#if 1  /* multi xfer test */
+#if 0  /* multi xfer test */
 	ret = multi_xfer_test(devh);
 	if(ret !=0) {
 		openusb_free_devid_list(devids);
@@ -1241,14 +1255,16 @@ int advance_xfer_test(void)
 #if 1	/* configuration test */
 	ret = openusb_set_configuration(devh,1);
 	if (ret != 0) {
-		printf("Set device(%llu) config error:%s\n",devids[0],
+		printf("Set device(%llu) config error:%s\n",
+				(long long)devids[0],
 				openusb_strerror(ret));
 		goto err;
 	}
 
 	ret = openusb_get_configuration(devh,&cfg);
 	if (ret != 0) {
-		printf("Get device(%llu) config error:%s\n",devids[0],
+		printf("Get device(%llu) config error:%s\n",
+				(long long)devids[0],
 				openusb_strerror(ret));
 		goto err;
 	}
@@ -1259,18 +1275,21 @@ int advance_xfer_test(void)
 #if 1	/* claim interface test */
 	ret = openusb_claim_interface(devh,0,0);
 	if (ret != 0) {
-		printf("Device(%llu) claim interface error:%s\n",devids[0],
+		printf("Device(%llu) claim interface error:%s\n",
+				(long long)devids[0],
 				openusb_strerror(ret));
 		goto err;
 	}
 
 	ret = openusb_is_interface_claimed(devh,0);
-	printf("Device(%llu) interface(0) claimed return %d\n", devids[0],
+	printf("Device(%llu) interface(0) claimed return %d\n",
+			(long long)devids[0],
 			ret);
 
 	ret = openusb_set_altsetting(devh,0,0);
 	if (ret != 0) {
-		printf("Device(%llu) interface(0) set alt:%s\n",devids[0],
+		printf("Device(%llu) interface(0) set alt:%s\n",
+				(long long)devids[0],
 				openusb_strerror(ret));
 		goto err;
 	}
@@ -1278,7 +1297,8 @@ int advance_xfer_test(void)
 
 	ret = openusb_get_altsetting(devh,0,&alt);
 	if (ret != 0) {
-		printf("Device(%llu) interface(0) get alt:%s\n",devids[0],
+		printf("Device(%llu) interface(0) get alt:%s\n",
+				(long long)devids[0],
 				openusb_strerror(ret));
 		goto err;
 	}
@@ -1286,7 +1306,8 @@ int advance_xfer_test(void)
 
 	ret = openusb_release_interface(devh,0);
 	if (ret != 0) {
-		printf("Device(%llu) release interface error:%s\n",devids[0],
+		printf("Device(%llu) release interface error:%s\n",
+				(long long)devids[0],
 				openusb_strerror(ret));
 		goto err;
 	}
@@ -1396,7 +1417,7 @@ int basic_test(void)
 		printf("error init\n");
 		exit(1);
 	}
-	printf("lib handle=%llu \n",libhandle);
+	printf("lib handle=%llu \n", (long long)libhandle);
 	printf("openusb_init PASS\n");
 
 	ret = openusb_set_event_callback(libhandle, USB_ATTACH, event_cb, NULL);
